import java.sql.SQLException;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

public abstract class AbstractCache {

	// Locks
	private final ReentrantReadWriteLock lock = new ReentrantReadWriteLock();
	private final Lock read = lock.readLock();
	private final Lock write = lock.writeLock();
	
	// Datenbankverbindung
	private final Database db;
	
	/**
	 * Erstelle Cache.
	 * @param db die offene Datenbankverbindung
	 * @throws SQLException bei SQL Fehler
	 */
	public AbstractCache(Database db) {
		this.db = db;
		
	}
	
	/**
	 * (Neues) Einlesen.
	 * @throws SQLException bei SQL Fehler
	 */
	public abstract void reload() throws SQLException;
	
	/**
	 * Datenbank zurückliefern.
	 */
	protected Database db() {
		return this.db;
	}
	
	/**
	 * Zum Schreiben sperren.
	 */
	protected void writeLock() {
		write.lock();
	}
	
	/**
	 * Zum Schreiben freigeben.
	 */
	protected void writeUnlock() {
		write.unlock();
	}
	
	/**
	 * Zum Lesen sperren.
	 */
	protected void readLock() {
		read.lock();
	}
	
	/**
	 * Zum Lesen freigeben.
	 */
	protected void readUnlock() {
		read.unlock();
	}
	
}
